import { getDictsDataByCodeArrary } from '@/api/common/dictionary';
import { useDictionary } from '@/store/modules/dictionary';
import type { DictCodeDataItem, DictData } from '@/types/DictionaryStore';
import { ToRefs } from 'vue';

type Options = {
  label: string;
  value: string | number;
};

type Result<T extends readonly string[]> = {
  [K in T[number]]?: Options[];
};

type EndResult<T extends readonly string[]> = {
  [K in T[number]]: Options[];
};

const formateDict = (data: DictCodeDataItem[]): Options[] => {
  const newData: Options[] = data.map((item) => {
    return {
      label: item.dictName,
      value: item.dictCode
    };
  });
  return newData;
};

export const useDictData = <T extends readonly string[]>(codes: T) => {
  const useDict = useDictionary();
  const result = reactive<Result<T>>({});
  const notAddedData: DictData = {};
  // 获取当前字典 sotre 内所有的key
  const dictsKey = Object.keys(useDict.dicts);
  // 找出传入项（codes）中不存在字典store内的 key
  const notAdded: string[] = [];
  codes.forEach((item) => {
    if (!item) return;
    if (dictsKey.includes(item)) {
      result[item] = formateDict(useDict.dicts[item]);
    } else {
      notAdded.push(item);
      result[item] = [];
      notAddedData[item] = [];
    }
  });
  if (!notAdded.length) return toRefs(result) as ToRefs<EndResult<T>>;
  const _data = [notAdded];
  _data.forEach(async (item) => {
    const { data } = await getDictsDataByCodeArrary(item);
    data.forEach((i) => {
      if (notAdded.includes(i.typeCode)) {
        notAddedData[i.typeCode].push({
          ...i
        });
        result[i.typeCode].push(...formateDict([i]));
      }
    });
    useDict.setDicts(notAddedData);
  });
  return toRefs(result) as ToRefs<EndResult<T>>;
};

/**
 *
 * @param dict 要查找的字典的整个数据数组
 * @param value 某个字典值的code
 * @returns 字典值的label
 * @description 根据字典值的value返回对应的label
 */
export const getDictLabel = (dict: Options[], value: string | number) => {
  const target = dict.find((item) => String(item.value) === String(value));
  return target ? target?.label : '';
};
